;
;------------------------------------------------------------------------------
;
; S H O W Q S
;
; Shows a Sinclair QL screen file (QS4 or QS8 for MODE 4 or 8) on EGA/VGA
; Started 91.12.11; this version: 92.02.11
;
; by Carlo Delhez, Emmastraat 3, 4651 BV Steenbergen, Netherlands
;
; via E-Mail on Internet:  tnndcarlo@cycl.phys.tue.nl  (until May 1994)
;
;------------------------------------------------------------------------------
; source suitable for the A86 assembler
;

org	100h				; COM file

	jmp start			; skip sys.var.

;------------------------------------------------------------------------------

error:	db 7,13,10
	db '!!! Command line error or no files found !!!',13,10
info:   db 13,10
	db 'SHOWQS: prints .QS4 or .QS8 QL screen files '
	db 'to EGA/VGA screen',13,10
	db 'Version 0.09, Copyright February 1992 by Carlo Delhez'
	db 13,10,13,10
	db 'Usage:    SHOWQS [filename or wildcard]',13,10
	db 'Default:  SHOWQS *.QS?',13,10,'$'

buffer: dw ?				; allocated buffer segment
colmod4:db 0,2,4,7			; black, green, red, white
colmod8:db 0,2,1,3,4,14,5,7		; blck,grn,blue,magnt,red,yllw,cyan,wht
file:	db '*.QS?',0,10 dup(?)          ; default wildcard & work space
flag:	dw ?				; count of matching files
palet:  db 0,1,2,3,4,5,6,7
	db 8,9,10,11,12,13,14,15
	db 0				; default set of colours & border
sto_1C: dw ?,?				; stores original Int 1Ch vector
sto_23: dw ?,?				; stores original Int 23h vector
ticks:	db ?				; timer tick countdown
tiep:	db ?				; stores QS-type ('4' or '8')
vinit:	db ?				; initial video mode

;------------------------------------------------------------------------------

start:
	mov bx,progend
	mov cl,4
	shr bx,cl
	inc bx
	mov ah,04Ah
	int 21h 			; tell DOS about my length

	xor bh,bh
	mov bl,cs:[80h] 		; fetch length of command
	cmp bl,0
	jz no_comm			; use default if no command supplied
	cmp b cs:[81h],' '
	jnz par_nok			; if not space first, error!
	cmp bl,1
	jz par_nok			; just 1 character not allowed
	dec bx
	cmp bl,13
	jc par_ok			; more than 13 characters not allowed
par_nok:
	jmp par_err			; display error
par_ok:
	mov b cs:[file+bx],0		; add Null at end of name
par_copy:
	mov ah,cs:[81h+bx]
	mov cs:[file+bx-1],ah
	dec bx
	jnz par_copy			; copy command to file definition
no_comm:
	mov ax,0351Ch			; read Int 1Ch vector > timer
	int 21h
	mov bp,sto_1C
	mov cs:[bp],es			; store segment...
	mov cs:[bp+2],bx		; ...and address in sto_1C
	mov ax,0251Ch			; set new Int 1Ch vector
	mov ds,cs
	mov dx,timer			; to ds:dx = timer
	int 21h

	mov ax,03523h			; read Int 23h vector > CTRL-C
	int 21h
	mov bp,sto_23
	mov cs:[bp],es			; store segment...
	mov cs:[bp+2],bx		; ...and address in sto_23
	mov ax,02523h			; set new Int 23h vector
	mov ds,cs
	mov dx,ctrl_c			; to ds:dx = ctrl_c
	int 21h

	mov ah,0Fh
	int 10h 			; determine current video mode
	mov cs:[vinit],al		; and store it for later use

	mov ax,0010h
	int 10h 			; set video mode 10h (page 0)

	mov ax,1002h
	mov es,cs
	mov dx,palet
	int 10h 			; set default palette

	mov bx,32*1024/16
	mov ah,48h
	int 21h 			; allocate 32k buffer
	mov cs:[buffer],ax		; store segment

forever:
	mov w cs:[flag],0		; number of files found

	mov ah,4Eh			; findfirst
	mov cx,0			; attribute
	mov ds,cs			; in code segment
	mov dx,file			; address of ASCIZ filename
	int 21h 			; findfirst (name at ds:dx)
	jnc ok1
	mov dx,error
	jmp fil_err			; if error, no files found: report
ok1:
	add w cs:[flag],1		; increment count
	mov ah,2Fh
	int 21h 			; find Disk Transfer Area in es:bx

	cmp w es:[bx+1Ah],32768
	jnz skip			; skip file if length != 32768
	cmp w es:[bx+1Ch],0
	jnz skip

	mov ds,es
	mov dx,bx
	add dx,1Eh			; ds:dx now points to name in DTA

	add bx,1Dh			; use bx to find end of ASCIZ name
find_z:
	inc bx
	cmp b es:[bx],0
	jnz find_z
	mov al,es:[bx-1]		; get last character of extension
        cmp al,'4'
	jz type_OK
	cmp al,'8'
	jz type_OK
	jmp skip			; skip if not '4' or '8'
type_OK:
	mov cs:[tiep],al		; store type

	mov ax,3D00h			; open_read file with DTA at ds:dx
	int 21h
	mov bx,ax			; handle in bx
	push bx
	mov ah,3Fh			; read from file
	mov cx,32768			; 32k bytes
	mov ds,cs:[buffer]
	mov dx,0			; into buffer
	int 21h

	call showqs			; print screen

	pop bx				; retrieve handle
	mov ah,3Eh
	int 21h 			; close file

	mov b cs:[ticks],182		; 10 seconds @ 18.2 Hz
await:
	mov ah,1			; do some keyboard checking
	int 16h
	jnz ctrl_c			; exit if key pressed
	cmp b cs:[ticks],0
	jnz await			; wait for countdown

skip:
	mov ah,4Fh
	int 21h 			; search next
	jnc ok1 			; repeat if no error

	cmp w cs:[flag],1		; only 1 file ?
	jz await			; then wait forever

	jmp forever			; otherwise start with 1st file again

par_err:
	mov ds,cs
	mov dx,error
	mov ah,9
	int 21h 			; print error to screen
	jmp exit
ctrl_c:
	mov dx,info			; set dx to print info only (next int!)
fil_err:
	mov ah,0
	mov al,cs:[vinit]		; retrieve initial video mode
	int 10h 			; and set video mode again

	mov ds,cs
	mov ah,9
	int 21h 			; print program info to screen (dx!)

	mov ax,0251Ch			; reset int 1Ch vector to old value
	mov bp,sto_1C			; stored in sto_1C
	mov ds,cs:[bp]
	mov dx,cs:[bp+2]
	int 21h 			; restore 18.2 Hz interrupt

	mov ax,02523h			; reset int 23h vector to old value
	mov bp,sto_23			; stored in sto_23
	mov ds,cs:[bp]
	mov dx,cs:[bp+2]
	int 21h 			; restore CTRL-C vector
exit:
	mov ax,4C00h
	int 21h 			; exit

;------------------------------------------------------------------------------

timer:	sub b cs:[ticks],1		; decrement `ticks' at 18.2 Hz
	iret

;------------------------------------------------------------------------------

showqs:
	mov ds,cs:[buffer]		; buffer segment
	mov bp,0			; start at address 0
	mov es,0A000h			; EGA screen segment
	mov dx,3CEh			; EGA port

	mov ax,0003
	out dx,ax
	mov ax,0205
	out dx,ax			; prepare EGA screen for output

	mov bx,80*(350-256)/2+(640-512)/16
					; centres picture (page 1: 32k further)
	mov ch,0			; 256 lines
loop4_y:
	mov cl,64			; 64 byte-pairs per line
loop4_x:
	push cx 			; store count
	mov cx,ds:[bp]			; get word from buffer
	inc bp
	inc bp				; update

	push bp
	call pix4			; write mode 4 or mode 8 screen
	pop bp

	inc bx				; next screen position
	pop cx				; retrieve count
	dec cl
	jnz loop4_x			; fill line
	add bx,80-64			; next line
	dec ch
	jnz loop4_y			; repeat for all lines

	ret

;------------------------------------------------------------------------------

pix4:
	cmp b cs:[tiep],'4'             ; use pix8 for a mode-8 picture
	jnz pix8

	mov dx,3CEh			; EGA port
	mov ax,8008h			; start with rightmost pixel output
eightpix:
	out dx,ax			; tell of which pixels to set colour
	push ax
	xor ax,ax
	rcl ch,1			; bit from 1st byte
	rcl al,1
	rcl cl,1			; and bit from 2nd byte
	rcl al,1			; get QL colour code in ax (0..3)
	mov bp,ax
	mov ah,cs:[colmod4+bp]		; use table to translate to EGA colour
	mov al,es:[bx]			; latch
	mov es:[bx],ah			; set new colour
	pop ax
	ror ah,1			; one pixel to the right
	jnc eightpix			; repeat 8 times
	ret

;------------------------------------------------------------------------------

pix8:
	mov dx,3CEh			; EGA port
	mov ax,0C008h			; start with 2 rightmost pixels
fourpix:
	out dx,ax			; tell of which pixels to set colour
	push ax
	xor ax,ax
	rcl ch,1			; 2 bits from 1st byte
	rcl al,1
	rcl ch,1
	rcl al,1
	rcl cl,1			; and bit from 2nd byte
	rcl al,1
	rcl cl,1			; neglect flash bit
	mov bp,ax
	mov ah,cs:[colmod8+bp]		; use table to translate to EGA colour
	mov al,es:[bx]			; latch
	mov es:[bx],ah			; set new colour
	pop ax
	ror ah,1
	ror ah,1			; two pixels to the right
	jnc fourpix			; repeat 4 times
	ret

;------------------------------------------------------------------------------

progend:

